<?php
// $Id: civicactions.patch.inc,v 1.1.2.1 2008/10/10 05:09:14 douggreen Exp $

/**
 * @file
 * CivicActions's Patch Page.
 */

function civicactions_patch_page() {
  if (function_exists('update_status_get_projects')) {
    $patches = array();
    // Retrieve the project information from the update_status module.
    $projects = update_status_get_projects();
    // Find the Drupal core patches.
    foreach (file_scan_directory("./", ".patch$", array('.', '..', 'CVS', 'sites')) as $file) {
      if (is_numeric($file->name)) {
        if (!isset($patches['drupal'])) {
          $patches['drupal'] = array('project' => $projects['drupal'], 'files' => array());
        }
        $patches['drupal']['files'][] = $file;
      }
    }
    // Find the contrib patches.
    $dirs = array();
    foreach ($projects as $project) {
      if ($project['name'] == 'drupal') {
        continue;
      }
      if ($path = _civicactions_get_project_path($project)) {
        foreach (file_scan_directory($path, ".patch$") as $file) {
          if (is_numeric($file->name)) {
            if (!isset($patches[$project['name']])) {
              $patches[$project['name']] = array('project' => $project, 'files' => array());
            }
            $file->title = _civicactions_get_patch_title($file->name);
            $patches[$project['name']]['files'][] = $file;
          }
        }
      }
    }
    return theme('civicactions_patches', $patches);
  }
  drupal_set_message(t('Update Status 2.x not installed'), 'error');
  return '';
}

function _civicactions_get_project_path($project) {
  $project_path = '';
  foreach (array_keys($project['modules']) as $module) {
    $module_path = drupal_get_path('module', $module);
    if (!$project_path || strlen($module_path) < strlen($project_path)) {
      $project_path = $module_path;
    }
  }
  return $project_path;
}

function _civicactions_get_patch_title($nid) {
  // First try and find the patch title from the cache.
  if ($cache = cache_get('civicactions_patch_info', 'cache')) {
    $titles = unserialize($cache->data);
    if (isset($titles[$nid])) {
      return $titles[$nid];
    }
  }
  // The title's not in the cache, so get the page from d.o.
  $html = drupal_http_request('http://drupal.org/node/'. $nid);
  if ($html->code == 200) {
    // We got a valid page from d.o, find the title.
    if ($title = _civicactions_parse_title($html->data)) {
      // Save the title in a cache so we don't need to do this too often.
      if (!isset($titles)) {
        $titles = array();
      }
      $titles[$nid] = $title;
      cache_set('civicactions_patch_info', 'cache', serialize($titles), time() + 86400);
      return $title;
    }
  }
}

function _civicactions_parse_title($html) {
  if (preg_match(',<title>(.*)</title>,', $html, $matches)) {
    return trim(preg_replace('/\|\s*drupal\.org$/', '', $matches[1]));
  }
}

function theme_civicactions_patches($patches) {
  if (count($patches)) {
    foreach ($patches as $path => $patch) {
      $rows[] = array(
        theme('civicactions_patch_project', $patch['project']),
        theme('civicactions_patch_files', $patch['files'])
      );
    }
    return theme('table', array(t('Project'), t('Patches')), $rows);
  }
  $output = t('No patches found.');
}

function theme_civicactions_patch_project($project) {
  return l($project['name'], 'http://drupal.org/project/'. $project['name']);
}

function theme_civicactions_patch_files($files) {
  $items = array();
  foreach ($files as $file) {
    $item = l('#'. $file->name, 'http://drupal.org/node/'. $file->name);
    if (!empty($file->title)) {
      $item .= ': '. check_plain($file->title);
    }
    $items[] = $item;
  }
  return implode('<br/>', $items);
}
